/*
 * ATMega8_IO_Port.c
 *
 * Created: 26.2.2015 12:37:47
 *  Author: marsik
 */ 


#include "ATMega8_IO-Port.h"
#include <avr/interrupt.h>
#include <util/delay.h>





int main(void)
{
  unsigned i = 0;

  Init();                      // inicializace celeho zarizeni

  while (1) {

	  if (i == COUNT_MAIN_CYCLES_SEND_MESSAGE) {
		  
		  SendStateMessage();      // odeslani zpravy o stavu zarizeni
		  i = 0;
		  
	  } else i++;

	  CheckSwitches();           // kontrola vstupu
	  CheckCommErrors();         // kontrola chyby parity, ramce a preteceni
	  SetDirection();            // nejdrive se nastavi spravny smer, pak se spusti generovani hodin
      SetFrequency();            // vyhodnoceni volby frekvence
  }
}






void Init() {

	DDRC = 0xFF;					//PORT C jako vystup
    PORTD = 0b00011100;				//PORT D jako vstupy - piny konfigurace frekvence s pull-up rezistory
	DDRB |= (1 << DDB1) | (1 << DDB2); //piny 1 a 2 portu B jako vystupy
	
	TCCR0 = (5 << CS00);			// Timer0 cita z preddelicky 1/1024
	TCCR1B |= (2 << CS10) | (1 << WGM12);	// preddelicka 1/8, CTC rezim s OCR1A (autoreload)
	TIMSK |= (1 << OCIE1A) | (1 << TOIE0);	// povoleni preruseni od T1 - Output compare a T0 - Overflow
	
	OCR1AH = OCR1AH_F5;
	OCR1AL = OCR1AL_F5;	

	USART_Init((unsigned int)MYUBRR);
//	_delay_ms(100);                 // Wait for UART module to stabilize
	
	UCSRB |= (1 << RXCIE);			// povoleni preruseni pri prijmu znaku
	
	SREG |= (1 << SREG_I);			// globalni povoleni preruseni

	USART_Transmit_Text("READY");
	USART_Transmit(13); USART_Transmit(10);

}


void USART_Init(unsigned int ubrr)
{

	UBRRH = (unsigned char)(ubrr>>8) | (1 << URSEL);
	UBRRL = (unsigned char)ubrr;

	UCSRC = (1<<URSEL)|(3<<UCSZ0)|(1<<UPM1);   // 8bit, even parity, 1 stop bit

	UCSRB = (1<<RXEN)|(1<<TXEN);			   // povoleni prijmu i vysilani

}


void USART_Transmit(unsigned char data) {

	while (!(UCSRA & (1<<UDRE)));
	
	UDR = data;

}


void USART_Transmit_Text(char *str) {
	
	uint8_t i = 0;
	
	while (((char)str[i]) != 0)
	{
		USART_Transmit((unsigned char)str[i++]);
	}
	
}


unsigned char USART_Receive()
{
	while (!(UCSRA & (1 << RXC)));
	return UDR;
}


void USART_Flush()
{
	unsigned char dummy;
	while (UCSRA & (1<<RXC)) dummy = UDR;
}

void SetFrequency()
{
	OCR1A_ready = 0;

	//switch (READ_MODE()) {  // dekodovani stavu konfig.pinu
	  switch (speed) {		//puvodne byla volba kmitoctu pomoci jumperu,
								//nyni se konfiguruje pres komunikaci
		case 0: OCR1AL_aux = OCR1AL_F0;
				OCR1AH_aux = OCR1AH_F0;
				break;

		case 1: OCR1AL_aux = OCR1AL_F1;
				OCR1AH_aux = OCR1AH_F1;
				break;

		case 2: OCR1AL_aux = OCR1AL_F2;
				OCR1AH_aux = OCR1AH_F2;
				break;

		case 3: OCR1AL_aux = OCR1AL_F3;
				OCR1AH_aux = OCR1AH_F3;
				break;

		case 4: OCR1AL_aux = OCR1AL_F4;
				OCR1AH_aux = OCR1AH_F4;
				break;

		case 5: OCR1AL_aux = OCR1AL_F5;
				OCR1AH_aux = OCR1AH_F5;
				break;

		case 6: OCR1AL_aux = OCR1AL_F6;
				OCR1AH_aux = OCR1AH_F6;
				break;

		case 7:
		case 8:
		case 9: OCR1AL_aux = OCR1AL_F7;
				OCR1AH_aux = OCR1AH_F7;
				break;
		default: ;
	  }

	OCR1A_ready = 1;
}


void SetDirection() {

	if (M[0].is_ready_to_set_direction) {     // takto kratke kusy kodu
		WRITE_M1DIR(M[0].direction);          // se nevyplati davat do cyklu
		_delay_ms(10);	
		M[0].run = 1;
		M[0].is_ready_to_set_direction = 0;
	}
	
	if (M[1].is_ready_to_set_direction) {
		WRITE_M2DIR(M[1].direction);
		_delay_ms(10);			
		M[1].run = 1;
		M[1].is_ready_to_set_direction = 0;
	}
	
}


void CheckSwitches() {

	M1switch.negative = (~READ_SW1N()) & 1;        // vstupy maji negativni logiku
	M1switch.positive = (~READ_SW1P()) & 1;
	M2switch.negative = (~READ_SW2N()) & 1;
	M2switch.positive = (~READ_SW2P()) & 1;

}


void CheckCommErrors() {

	if (comm_error.parity) {
		USART_Transmit_Text("ERR_C_P"); USART_Transmit(13); USART_Transmit(10);
		comm_error.parity = 0;
	}
	
	if (comm_error.frame) {
		USART_Transmit_Text("ERR_C_F"); USART_Transmit(13); USART_Transmit(10);
		comm_error.frame = 0;
	}
	
	if (comm_error.overrun) {
		USART_Transmit_Text("ERR_C_O");  USART_Transmit(13); USART_Transmit(10);
		comm_error.overrun = 0;
	}
	
}


void SendStateMessage() {

	char SpeedMessage[5] = "SPD";
	char M1Message[18] = "M1";
	char M2Message[18] = "M2";
	
	
	
	SpeedMessage[3] = speed + '0';
	
	USART_Transmit_Text(SpeedMessage);
	USART_Transmit(13);USART_Transmit(10);	
	

	//    M1Message[0] = 'M';
	//    M1Message[1] = '1';
	if (READ_DOUT1()) M1Message[2]='H'; else M1Message[2]='L';
	if (M1switch.negative) M1Message[3]='N'; else M1Message[3]='_';
	if (M1switch.positive) M1Message[4]='P'; else M1Message[4]='_';

	if (M[0].run)
	if (M[0].manual) M1Message[5]='J'; else M1Message[5]='P';
	else M1Message[5]='_';


	LTOA_WithZeroes(M[0].act_steps, M1Message+6);

	USART_Transmit_Text(M1Message);
	USART_Transmit(13);USART_Transmit(10);


	//    M2Message[0] = 'M';
	//    M2Message[1] = '2';
	if (READ_DOUT2()) M2Message[2]='H'; else M2Message[2]='L';
	if (M2switch.negative) M2Message[3]='N'; else M2Message[3]='_';
	if (M2switch.positive) M2Message[4]='P'; else M2Message[4]='_';
	
	if (M[1].run)
	if (M[1].manual) M2Message[5]='J'; else M2Message[5]='P';
	else M2Message[5]='_';


	LTOA_WithZeroes(M[1].act_steps, M2Message+6);

	USART_Transmit_Text(M2Message);
	USART_Transmit(13);USART_Transmit(10);
	
	USART_Transmit(13);USART_Transmit(10);
	
}


void LTOA_WithZeroes(long x, char *str) {

	str[0] = x < 0 ? '-' : '+';			
	
	x = x < 0 ? -x : x;

	for (int i=10; i>0; i--) {
		
		str[i] = x % 10  + '0';
		x /= 10;
		
	}
	
	str[11] = 0;
	
}


void ReceiveControlMessage() {

	char uart_rd;
	static unsigned int index, im;
	static struct movement Mtemp[2];
	static char first_run = 5;
	char AliveCommand[6] = "ALIVE";
	

	uart_rd = USART_Receive();      // read the received data,

	if ((UCSRA & 0b00011100) && !first_run) {    //kontrola chyb prenosu (po resetu se prvni znaky nekontroluji)
		index = 0;
		WRITE_LED1(1);
		M[0].stop = 1;
		M[1].stop = 1;
		if ((UCSRA & 0b00000100)>>2) comm_error.parity = 1;
		if ((UCSRA & 0b00001000)>>3) comm_error.overrun = 1;
		if ((UCSRA & 0b00010000)>>4) comm_error.frame = 1;
		
		USART_Flush();
		return;
	}
	else {

		WRITE_LED1(0);
		
		if (index == 100) {
			index = 0;
			switch (uart_rd) {
				case 'S':  if (im==0) WRITE_DOUT1(1);
						   if (im==1) WRITE_DOUT2(1);
				return;
				//break;

				case 'R':  if (im==0) WRITE_DOUT1(0);
						   if (im==1) WRITE_DOUT2(0);
				return;
				//break;
				
				default:   ;
				return;
			}
		}
		
		
		if ((index >= 3) && (index <= 11)) {
			if (((uart_rd >= '0') && (uart_rd <= '9')) || (uart_rd == 'J')) {

				if ((index == 3) && (uart_rd == 'J')) {
					if (!M[im].run && !M[im].is_ready_to_set_direction) {
						
						M[im].direction = Mtemp[im].direction;
						M[im].manual = 1;
						M[im].is_ready_to_set_direction = 1;
						M[im].stop = 0;
						
					}
					
					if (M[im].run && M[im].manual && (M[im].direction == Mtemp[im].direction)) timeout_counter[im] = 0;
					
					index = 0;
					
					} else if (uart_rd != 'J') {
						
						if (index == 3) Mtemp[im].steps = 0;
						Mtemp[im].steps = Mtemp[im].steps*10 + (uart_rd - '0');

						if (index == 11) {
							
							index = 0;
							
							if (!M[im].run && !M[im].is_ready_to_set_direction) {
								
								M[im].set_steps = Mtemp[im].steps;
								M[im].direction = Mtemp[im].direction;
								M[im].manual = 0;
								M[im].is_ready_to_set_direction = Mtemp[im].steps != 0;
								M[im].stop = 0;
								
							}
							
						} else index++;
						
					} else index = 0;

			} else index = 0;

			return;
		}


		if (index == 2) {
			index = 3;
			switch (uart_rd) {
				case '-': Mtemp[im].direction = 0;
				return;
				//break;

				case '+': Mtemp[im].direction = 1;
				return;
				//break;

				case 'R':	M[im].act_steps = 0;
							index = 0;
				return;
				//break;

				case 'S': M[im].stop = 1;
				index = 0;
				return;
				//break;
				
				case 'D': index = 100;
				return;
				//break;

				default: index = 0;
				return;
			}
		}


		if (index == 1) {
			index = 2;
			switch (uart_rd) {
				case '1':  im = 0;
				return;
				//break;

				case '2':  im = 1;
				return;
				//break;
				
				default:   index = 0;
				return;
			}
		}


		if ((uart_rd == 'M') && (index == 0)) {
			index = 1;
			//return;
		}
		
		
		if (index == 103) {
			if ((uart_rd>='0') && (uart_rd<='9')) speed = uart_rd - '0';
			index = 0;
		}

				
		if (index == 102) {
			if (uart_rd == 'D') index = 103;
			else index = 0;
		}		

		
		if (index == 101) {
			if (uart_rd == 'P') index = 102;
			else index = 0;
		}
				
	
		if ((uart_rd == 'S') && (index == 0)){
			index = 101;
		}
		
		
		if (index >= 201) {
			if (uart_rd == AliveCommand[index-200]) index++;
			else index = 0;
					
			if (index == 205) {
				timeout_counter[0] = 0;
				timeout_counter[1] = 0;
				index = 0;
			}
		}
		
		if ((uart_rd == AliveCommand[0]) && (index == 0)){
			index = 201;
		}
	}

	if (first_run) first_run--;
}


void GenerateClockSignal() {

	static long Msteps_temp[2];
	static char Msteps_counting[2];
	static char PIN_MCLK[2];
	static char PIN_MDIR[2];
	static char PIN_MCLK_old[2];


	if (OCR1A_ready) {	// nastav spravne citac podle zadane rychlosti
		
		OCR1AH = OCR1AH_aux;
		OCR1AL = OCR1AL_aux;
		
	}
	

	PIN_MDIR[0] = READ_M1DIR();
	PIN_MDIR[1] = READ_M2DIR();

	for (int i=0; i<=1; i++) {

		if (!M[i].stop) {
			if (M[i].run) {
				
				if (M[i].manual) PIN_MCLK[i] ^= 1;       // negace nulteho bitu (vsechny ostatni musi byt vynulovane)
				else if (!Msteps_counting[i]) {
					
					Msteps_counting[i] = 1;
					Msteps_temp[i] = 2*M[i].set_steps;
					
					} else {
						
						PIN_MCLK[i] ^= 1;					 // negace nulteho bitu (vsechny ostatni musi byt vynulovane)
						Msteps_temp[i]--;
						if (Msteps_temp[i] == 0) {

							Msteps_counting[i] = 0;
							M[i].run = 0;
							PIN_MCLK[i] = 0;

						}
						
					}
			}
		} else {
			Msteps_counting[i] = 0;
			M[i].run = 0;
			PIN_MCLK[i] = 0;
		}
		

		if (PIN_MCLK[i] && !PIN_MCLK_old[i]) {    // citani NABEZNYCH hran - kroku motoru

			if (PIN_MDIR[i]) M[i].act_steps++; else M[i].act_steps--;

		}
		
		PIN_MCLK_old[i] = PIN_MCLK[i];
	}
	
	WRITE_M1CLK(PIN_MCLK[0]);
	WRITE_M2CLK(PIN_MCLK[1]);
}


ISR(USART_RXC_vect) {

	ReceiveControlMessage();
	
}


ISR(TIMER1_COMPA_vect) {
	
    GenerateClockSignal();

    WRITE_LED2(M[0].run | M[1].run);	

}


ISR(TIMER0_OVF_vect) {
	
	if (timeout_counter[0] == TIMEOUT_x107ms) {
		
		if (M[0].run) M[0].stop = 1;
		timeout_counter[0] = 0;

	} else timeout_counter[0]++;
	
	
	if (timeout_counter[1] == TIMEOUT_x107ms) {
		
		if (M[1].run) M[1].stop = 1;
		timeout_counter[1] = 0;

	} else timeout_counter[1]++;
	
}