#include <avr/io.h>

// PRIKAZY:
// M1-999999999
// M1+999999999
// M1-J
// M1+J
// M1R
// M1S
// M1DS
// M1DR
// M2-999999999
// M2+999999999
// M2-J
// M2+J
// M2R
// M2S
// M2DS
// M2DR



// VYSTUPY
#define WRITE_M1CLK(x)  (PORTC = (PORTC & ~_BV(PC3)) | ((x) << PC3))
#define WRITE_M2CLK(x)  (PORTC = (PORTC & ~_BV(PC1)) | ((x) << PC1))
#define WRITE_M1DIR(x)  (PORTC = (PORTC & ~_BV(PC2)) | ((x) << PC2))
#define WRITE_M2DIR(x)  (PORTC = (PORTC & ~_BV(PC0)) | ((x) << PC0))

#define WRITE_DOUT1(x)  (PORTB = (PORTB & ~_BV(PB2)) | ((x) << PB2))
#define WRITE_DOUT2(x)  (PORTB = (PORTB & ~_BV(PB1)) | ((x) << PB1))

#define WRITE_LED1(x)  (PORTC = (PORTC & ~_BV(PC5)) | ((x) << PC5))
#define WRITE_LED2(x)  (PORTC = (PORTC & ~_BV(PC4)) | ((x) << PC4))

#define READ_M1DIR()    ((PORTC & _BV(PC2)) >> PC2)
#define READ_M2DIR()    ((PORTC & _BV(PC0)) >> PC0)

#define READ_DOUT1()  ((PORTB & _BV(PB2)) >> PB2)
#define READ_DOUT2()  ((PORTB & _BV(PB1)) >> PB1)



// VSTUPY
#define READ_SW1N()    ((PIND & _BV(PIND5)) >> PIND5)
#define READ_SW1P()    ((PIND & _BV(PIND6)) >> PIND6)
#define READ_SW2N()    ((PIND & _BV(PIND7)) >> PIND7)
#define READ_SW2P()    ((PINB & _BV(PINB0)) >> PINB0)

#define READ_MODE()	   ((PIND & (_BV(PIND2) | _BV(PIND3) | _BV(PIND4))) >> PIND2)    //puvodne pouzito pro volbu kmitoctu
																					 //nyni volne k pouziti



// OSTATNI
#define F_CPU 2457600UL		// taktovaci frekvence MCU v Hz
#define COUNT_MAIN_CYCLES_SEND_MESSAGE (F_CPU/1000)// pocet cyklu hlavni smycky pro odeslani stavove zpravy
#define BAUD 19200			// pro 38400Bd uz dochazi k preteceni pri X-tal 2,4576MHz
#define MYUBRR (F_CPU/16UL/BAUD-1UL)
#define DIV 8				// hodnota delicky frekvence pro CTC rezim Timer1
#define TIMEOUT_x107ms 14	// definuje cas timeoutu manualniho rezimu cca 1,5s
#define SPEED_INIT 5		// cislo vystupni frekvence po startu MCU


#define F0 125             // x0.1Hz => 12.5Hz
#define OCR1AL_F0 ((10*F_CPU/(F0*DIV*2))%256)
#define OCR1AH_F0 ((10*F_CPU/(F0*DIV*2))/256)

#define F1 25             // Hz
#define OCR1AL_F1 ((F_CPU/(F1*DIV*2))%256)
#define OCR1AH_F1 ((F_CPU/(F1*DIV*2))/256)

#define F2 50             // Hz
#define OCR1AL_F2 ((F_CPU/(F2*DIV*2))%256)
#define OCR1AH_F2 ((F_CPU/(F2*DIV*2))/256)

#define F3 100             // Hz
#define OCR1AL_F3 ((F_CPU/(F3*DIV*2))%256)
#define OCR1AH_F3 ((F_CPU/(F3*DIV*2))/256)

#define F4 200             // Hz
#define OCR1AL_F4 ((F_CPU/(F4*DIV*2))%256)
#define OCR1AH_F4 ((F_CPU/(F4*DIV*2))/256)

#define F5 400             // Hz
#define OCR1AL_F5 ((F_CPU/(F5*DIV*2))%256)
#define OCR1AH_F5 ((F_CPU/(F5*DIV*2))/256)

#define F6 800             // Hz
#define OCR1AL_F6 ((F_CPU/(F6*DIV*2))%256)
#define OCR1AH_F6 ((F_CPU/(F6*DIV*2))/256)

#define F7 1600            // Hz
#define OCR1AL_F7 ((F_CPU/(F7*DIV*2))%256)
#define OCR1AH_F7 ((F_CPU/(F7*DIV*2))/256)






struct movement
{						           // pomocna struktura pro prijem
	char direction;                // ridici zpravy, ktera se nevyhodnocuje
	long steps;					   // v hlavni smycce, tudiz nemusi byt
};								   // volatile


volatile struct motor
{                      // ridici struktura motoru

	char manual;
	char direction;
	char stop;
	long set_steps;
	long act_steps;
	char is_ready_to_set_direction;   // nastavuje ReceiveControlMessage()
	// resetuje SetDirection()
	char run;                         // nastavuje ReceiveControlMessage()
	// resetuje GenerateClockSignal()
	
} M[2] = {{0, 0}};


struct {                            // struktura paru snimacu
	char negative;
	char positive;
} M1switch = {0}, M2switch = {0};


volatile struct {                      // hlaseni komunikacnich chyb
	char parity;
	char frame;
	char overrun;
} comm_error = {0};


volatile int timeout_counter[2] = {0,0};		// promenne timeoutu - snulovanim se resetuje timeout
												// urceno pro rucni rezim (vyhodnocuje se v ISR Timer0)
char speed = SPEED_INIT;			// cislo vystupni frekvence
char OCR1A_ready;					// synchronizacni promenna
int OCR1AL_aux;						// pomocne promenne
int OCR1AH_aux;



void Init();
void USART_Init();
void USART_Flush();
void USART_Transmit(unsigned char data);
void USART_Transmit_Text(char *str);
unsigned char USART_Receive();
void CheckCommErrors();
void SendStateMessage();
void CheckSwitches();
void SetFrequency();
void SetDirection();
void LTOA_WithZeroes(long x, char *str);